'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _initStorage = require('./initStorage');

var _initStorage2 = _interopRequireDefault(_initStorage);

var _getIndexedDB = require('./utils/getIndexedDB');

var _getIndexedDB2 = _interopRequireDefault(_getIndexedDB);

var _constants = require('./constants');

var _indexeddb = require('./drivers/indexeddb');

var _indexeddb2 = _interopRequireDefault(_indexeddb);

var _memory = require('./drivers/memory');

var _memory2 = _interopRequireDefault(_memory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var indexedDB = (0, _getIndexedDB2.default)();

function createInstance(_ref) {
  var storeName = _ref.storeName,
      _ref$version = _ref.version,
      version = _ref$version === undefined ? _constants.DEFAULT_VERSION : _ref$version,
      _ref$dbName = _ref.dbName,
      dbName = _ref$dbName === undefined ? _constants.DEFAULT_DB_NAME : _ref$dbName,
      _ref$options = _ref.options,
      options = _ref$options === undefined ? {} : _ref$options;

  var memoryStorage = {};

  if (!indexedDB) {
    throw new Error('No indexedDB implementation found!');
  }

  if (!storeName) {
    throw new Error('No store name provided for Indexed DB instance.');
  }

  if (!dbName) {
    throw new Error('No database name provided for Indexed DB instance.');
  }

  var databaseReadyPromise = (0, _initStorage2.default)({ dbName: dbName, storeName: storeName, version: version, options: options }).then(function (dbInstance) {
    return (0, _indexeddb2.default)(dbInstance);
  }).catch(function (error) {
    var name = error.name;
    // `InvalidStateError` occurs in Firefox private mode.
    // `SecurityError` occurs in Safari.
    // When this happens we need to use an in-memory storage solution
    // instead of relying of IndexedDB.

    if (name && name === 'InvalidStateError' || name === 'SecurityError') {
      return (0, _memory2.default)(memoryStorage);
    }

    throw new Error(error);
  });

  var getItem = function getItem(key) {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.getItem(storeName, key);
    });
  };

  var setItem = function setItem(key, value) {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.setItem(storeName, key, value);
    });
  };

  var clear = function clear() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.clear(storeName);
    });
  };

  var close = function close() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.close(dbName, options);
    });
  };

  var getCollection = function getCollection() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.getCollection(storeName);
    });
  };

  var removeItem = function removeItem(key) {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.removeItem(storeName, key);
    });
  };

  var length = function length() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.length(storeName);
    });
  };

  var keys = function keys() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.keys(storeName);
    });
  };

  var iterate = function iterate(callback) {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.iterate(storeName, callback);
    });
  };

  var destroy = function destroy() {
    return databaseReadyPromise.then(function (dbInstance) {
      return dbInstance.destroy(dbName, options);
    });
  };

  return {
    getItem: getItem,
    setItem: setItem,
    clear: clear,
    close: close,
    getCollection: getCollection,
    removeItem: removeItem,
    length: length,
    keys: keys,
    iterate: iterate,
    destroy: destroy
  };
}

exports.default = createInstance;