'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _constants = require('../constants');

var _destroyDatabase = require('../utils/destroyDatabase');

var _destroyDatabase2 = _interopRequireDefault(_destroyDatabase);

var _validateKey = require('../utils/validateKey');

var _validateKey2 = _interopRequireDefault(_validateKey);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createIndexedDBDriver = function createIndexedDBDriver(dbInstance) {
  // Retrieve an item from the desired object store under the provided key
  var getItem = function getItem(name, key) {
    key = (0, _validateKey2.default)(key);

    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
      var objectStore = transaction.objectStore(name);

      var req = objectStore.get(key);

      req.onsuccess = function () {
        return resolve(req.result);
      };
      req.onabort = function () {
        return reject(transaction);
      };
      req.onerror = function () {
        return reject(transaction);
      };
    });
  };

  // Store an item in the provided store name under the provided key
  var setItem = function setItem(name, key, value) {
    key = (0, _validateKey2.default)(key);

    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
      var objectStore = transaction.objectStore(name);

      // Saving as `null` will result in a subsequent getItem returning `undefined`
      // Make it `undefined` immediately so that there is consistency
      if (value === null) {
        value = undefined;
      }

      objectStore.put(value, key);

      transaction.oncomplete = function () {
        return resolve(value, key);
      };
      transaction.onabort = function () {
        return reject(transaction);
      };
      transaction.onerror = function () {
        return reject(transaction);
      };
    });
  };

  // Return all the items stored in the provided store name
  var getCollection = function getCollection(name) {
    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
      var objectStore = transaction.objectStore(name);

      var req = objectStore.getAll();

      req.onsuccess = function () {
        return resolve(req.result);
      };
      req.onabort = function () {
        return reject(req);
      };
      req.onerror = function () {
        return reject(req);
      };
    });
  };

  // Remove an item from the provided store name under the provided key
  var removeItem = function removeItem(name, key) {
    key = (0, _validateKey2.default)(key);

    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
      var objectStore = transaction.objectStore(name);

      objectStore.delete(key);

      transaction.oncomplete = function () {
        return resolve();
      };
      transaction.onabort = function () {
        return reject(transaction);
      };
      transaction.onerror = function () {
        return reject(transaction);
      };
    });
  };

  var clear = function clear(name) {
    return new Promise(function (resolve, reject) {
      try {
        var transaction = dbInstance.transaction(name, _constants.READ_WRITE);
        var objectStore = transaction.objectStore(name);

        var req = objectStore.clear();

        var onError = function onError() {
          var err = req.error ? req.error : req.transaction.error;
          reject(err);
        };

        transaction.oncomplete = function () {
          return resolve();
        };
        transaction.onabort = onError;
        transaction.onerror = onError;
      } catch (err) {
        reject({
          storeName: name,
          error: err
        });
      }
    });
  };

  // Get the size of the provided store name
  var length = function length(name) {
    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
      var objectStore = transaction.objectStore(name);

      var req = objectStore.count();

      req.onsuccess = function () {
        return resolve(req.result);
      };
      req.onabort = function () {
        return reject(req);
      };
      req.onerror = function () {
        return reject(req);
      };
    });
  };

  // Return all of the keys in the object store
  var keys = function keys(name) {
    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
      var objectStore = transaction.objectStore(name);

      try {
        var req = objectStore.openCursor();
        var storeKeys = [];

        req.onsuccess = function () {
          var cursor = req.result;

          // If the cursor is empty, send back the keys
          if (!cursor) {
            resolve(storeKeys);
            return;
          }

          // Otherwise keep going
          storeKeys.push(cursor.key);
          cursor.continue();
        };

        req.onerror = function () {
          return reject(req.error);
        };
      } catch (e) {
        reject(e);
      }
    });
  };

  var iterate = function iterate(name, callback) {
    return new Promise(function (resolve, reject) {
      var transaction = dbInstance.transaction(name, _constants.READ_ONLY);
      var objectStore = transaction.objectStore(name);

      try {
        var req = objectStore.openCursor();

        req.onsuccess = function () {
          var cursor = req.result;

          // If the cursor is empty, send back the keys
          if (!cursor) {
            resolve();
            return;
          }

          // Otherwise keep going
          callback(cursor.value, cursor.key, cursor);
          cursor.continue();
        };

        req.onerror = function () {
          return reject(req.error);
        };
      } catch (e) {
        reject(e);
      }
    });
  };

  // Kills the connection to the DB
  var close = function close(dbName, options) {
    return new Promise(function (resolve) {
      dbInstance.close();

      if (options.enableLogging) {
        console.info('Closed connection to: ', dbName);
      }

      resolve();
    });
  };

  // Destroys the database instance
  var destroy = function destroy(dbName, options) {
    dbInstance.close();

    return (0, _destroyDatabase2.default)(dbName, options);
  };

  return {
    getItem: getItem,
    setItem: setItem,
    getCollection: getCollection,
    removeItem: removeItem,
    clear: clear,
    length: length,
    keys: keys,
    iterate: iterate,
    close: close,
    destroy: destroy
  };
};

exports.default = createIndexedDBDriver;