'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIndexedDB = require('./utils/getIndexedDB');

var _getIndexedDB2 = _interopRequireDefault(_getIndexedDB);

var _constants = require('./constants');

var _clearStore = require('./clearStore');

var _clearStore2 = _interopRequireDefault(_clearStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultOptions = {
  autoIncrement: true,
  disableContexts: false // Allow the ability to disable contexts for testing
};

var indexedDB = (0, _getIndexedDB2.default)();

var dbContexts = {};

var isUpgradeRequired = function isUpgradeRequired(dbInfo) {
  if (!dbInfo.db) {
    return true;
  }

  var isNewStore = !dbInfo.db.objectStoreNames.contains(dbInfo.storeName);
  var isUpgrade = dbInfo.version > dbInfo.db.version;
  var isDowngrade = dbInfo.version < dbInfo.db.version;

  if (isNewStore) {
    var nextVersion = dbInfo.db.version + 1;

    if (nextVersion > dbInfo.version) {
      dbInfo.version = nextVersion;
    }

    return true;
  }

  if (isUpgrade) {
    return true;
  }

  // Downgrades are not possible, align the versions to avoid errors
  if (isDowngrade) {
    if (dbInfo.version !== _constants.DEFAULT_VERSION) {
      console.warn('Database ' + dbInfo.name + ' has been downgraded. Aligning versions to avoid errors.');
    }

    dbInfo.version = dbInfo.db.version;
  }

  return false;
};

var getConnection = function getConnection(_ref, forceUpgrade) {
  var name = _ref.name,
      version = _ref.version,
      db = _ref.db,
      storeName = _ref.storeName;

  return new Promise(function (resolve, reject) {
    var shouldClear = false;

    if (db && !forceUpgrade) {
      resolve(db);
      return;
    }

    if (db && forceUpgrade) {
      db.close();
    }

    try {
      var dbConnection = indexedDB.open(name, version);

      // An upgrade needs to be performed, clear out the old object stores
      // and then create the new object store
      dbConnection.onupgradeneeded = function (evt) {
        var oldVersion = evt.oldVersion,
            newVersion = evt.newVersion;

        var dbInstance = dbConnection.result;

        try {
          dbInstance.createObjectStore(storeName);
        } catch (err) {
          if (err.name === 'ConstraintError') {
            console.warn('Database ' + name + ' has been upgraded from ' + oldVersion + ' -> ' + newVersion + ' but the table ' + storeName + ' already exists.');

            // Force the store to clear because we've incremented the version
            shouldClear = true;
          } else {
            throw err;
          }
        }
      };

      // An error occured attempting to connect to the DB
      dbConnection.onerror = function (evt) {
        // See https://mzl.la/2wCPech
        evt.preventDefault();

        reject(evt.target.error);
      };

      // Successfully connected to the database
      dbConnection.onsuccess = function (evt) {
        var dbInstance = evt.target.result;

        if (shouldClear) {
          resolve((0, _clearStore2.default)(dbInstance, storeName));
          return;
        }

        resolve(dbInstance);
        return;
      };
    } catch (e) {
      reject(e);
    }

    return;
  });
};

var initStorage = function initStorage(_ref2) {
  var dbName = _ref2.dbName,
      version = _ref2.version,
      storeName = _ref2.storeName,
      options = _ref2.options;

  // Get the context of the provided dbName
  var dbInfo = dbContexts[dbName];
  var opts = Object.assign({}, defaultOptions, options);

  // Or initialize one if it doesn't exist
  if (!dbInfo) {
    dbInfo = {
      db: null,
      name: dbName,
      version: version,
      storeName: storeName,
      options: opts
    };

    if (!options.disableContexts) {
      // Register the new context
      dbContexts[dbName] = dbInfo;
    }
  }

  // Get the database connection, if we need to upgrade it then perform the
  // required steps.
  return getConnection(dbInfo, false).then(function (db) {
    dbInfo.db = db;

    if (isUpgradeRequired(dbInfo)) {
      return getConnection(dbInfo, true);
    }

    return db;
  });
};

exports.default = initStorage;